//---------------------------------------------------------------------------
//      DS24B33
//---------------------------------------------------------------------------
#pragma hdrstop
#include "DS24B33.h"
#include "ProgressBar.h"        //   -
#include "MainProgram.h"        //    
#include "ChipFunctions.h"      //      
#include "PortFunctions.h"      //    
#include "CalcCRC.h"            //   CRC
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0x0F) (  1, 2) -------------------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.ES, Input.Size, Input.AreaNumber
// output:      Test - ,      (0xFF) + Success
DS24B33_OUTPUT_DATA DS24B33_WriteToScratchpad(DS24B33_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned short int CRC16;             //   CRC16
  DS24B33_OUTPUT_DATA OutputData;       //   
  unsigned int Address;                 //   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //   
  Address=InputData.TA1+0xFF*InputData.TA2;             //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Write Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x0F;                                  // Write Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  //     CRC16
  CRC16=0; CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);      //   CRC16   Write Scratchpad
  Sleep(10);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  // TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);               //   CRC16
  Sleep(1);
  //***     ***
  for(unsigned int i=0; i<InputData.Size; i++)          //   ,   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    switch(InputData.AreaNumber)
    { case 1:   wr_buf.Data[1]=SelectedChip.Area1[Address-SelectedChip.Area1Start];     //    
                break;
      case 2:   wr_buf.Data[1]=SelectedChip.Area2[i];                                   //   
                break;
      default:  return OutputData;
    }
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    CRC16=CalcCRC16(CRC16, wr_buf.Data[1]);             //   CRC16
    Address+=1;                                         //  
    LBar->ProgressBar1->Position=100*(i+1)/InputData.Size;      //    -
    Sleep(1);
  }
  //  CRC16 (2 , )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];                       // CRC16    
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   (  CRC16)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  rd_buf.Data[0]=~rd_buf.Data[0];                       // CRC16    
  CRC16=CalcCRC16(CRC16, rd_buf.Data[0]);               //   CRC16
  Sleep(1);
  //   CRC16
  if(CRC16)     SendMsgToLogForm(16,"");          //  CRC16  //    -     
  // ,     (  ,    1-)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;          //    Test   
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0xAA) (  Area2) --------------------------
//---------------------------------------------------------------------------
// output:      Test - ,      (0xFF) + TA1 + TA2 + ES + Success
DS24B33_OUTPUT_DATA DS24B33_ReadFromScratchpad()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS24B33_OUTPUT_DATA OutputData;       //   
  unsigned int Address;           	//   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Scratchpad-
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xAA;                                  // Read Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //*** READ TA1, TA2, E/S ***
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA1
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.TA1=rd_buf.Data[0];                        //  TA1
  Sleep(1);
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.TA2=rd_buf.Data[0];                        //  TA2
  Sleep(1);
  Address=0;                                            //        TA1, TA2
  //   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  // ReadByte_1W
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  ES
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.ES=rd_buf.Data[0];                         //  ES
  Sleep(1);
  //***    ***
  for(unsigned int i=0; i<SelectedChip.Area2Size; i++)  //   
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                              // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    SelectedChip.Area2[i]=rd_buf.Data[0];               //      
    Address+=1;                                                         //  
    LBar->ProgressBar1->Position=100*(i+1)/SelectedChip.Area2Size;      //    -
    Sleep(1);
  }
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD  EEPROM (0x55) -----------------------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2, Input.ES
// output:      Test - ,      (0xAA) + Success
DS24B33_OUTPUT_DATA DS24B33_CopyScratchpadToMemory(DS24B33_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS24B33_OUTPUT_DATA OutputData;        //   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Copy Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Copy Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  E/S
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.ES;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  // ,    
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---    (0xF0) (   Area1) ----------------------
//---------------------------------------------------------------------------
// input:       Input.TA1, Input.TA2
// output:      Test - ,      (0xAA) + Success
DS24B33_OUTPUT_DATA DS24B33_ReadFromMemory(DS24B33_INPUT_DATA InputData)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS24B33_OUTPUT_DATA OutputData;       //   
  unsigned int Address;           	//   

  OutputData.TA1=0;
  OutputData.TA2=0;
  OutputData.ES=0;
  OutputData.Test=0;
  OutputData.Success=false;             //  
  Address=InputData.TA1+0xFF*InputData.TA2;             //    ,    

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //   EEPROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xF0;                                  // Read EEPROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //  TA1
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA1;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  TA2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=InputData.TA2;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //***    ***
  for(unsigned int i=0; i<SelectedChip.Area1Size; i++)  //   
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                           LBar->Free();                //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                                  // 
    if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    SelectedChip.Area1[Address-SelectedChip.Area1Start]=rd_buf.Data[0];         //      
    Sleep(1);
    Address+=1;                                                         //  

    LBar->ProgressBar1->Position=100*(i+1)/SelectedChip.Area1Size;      //    -
    Sleep(1);
  }
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                    // 
  if(!rd_buf.Size) { SendMsgToLogForm(21,"");          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11,"");          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//--------------------------------------------------------------------------- 
